var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var REQUEST_HEADERS = {
    form: { 'Content-Type': 'application/x-www-form-urlencoded' },
    json: { 'Content-Type': 'application/json;charset=utf-8' },
};
/**
 * format request path
 * checking on https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Template_literals
 * @param params request params
 * @param strings template literals string array
 * @param keys template literals key array
 * @returns formatted string
 */
function formatRequestPath(params, strings, keys) {
    var result = [strings[0]];
    keys.forEach(function (key, i) {
        var value = typeof key === 'function' ? key(params) : params[key];
        result.push(value, strings[i + 1]);
    });
    return result.join('');
}
export function typedUrl(configOrStrings) {
    var keys = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        keys[_i - 1] = arguments[_i];
    }
    // withConfig
    if (!Array.isArray(configOrStrings)) {
        return function (strings) {
            var innerKeys = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                innerKeys[_i - 1] = arguments[_i];
            }
            return function (params) {
                return [configOrStrings, formatRequestPath(params, strings, innerKeys)];
            };
        };
    }
    else {
        return function (params) {
            return formatRequestPath(params, configOrStrings, keys);
        };
    }
}
export function registApi(client, apis, prefix, id) {
    var result = {};
    Object.keys(apis).forEach(function (methodName) {
        var methodUrl = apis[methodName];
        result[methodName] = transfromToRequest(client, methodUrl, prefix, id);
    });
    return result;
}
function transfromToRequest(request, methodUrl, prefix, id) {
    if (prefix === void 0) { prefix = ''; }
    return function (config) {
        if (config === void 0) { config = {}; }
        var _a = config.requestType, requestType = _a === void 0 ? 'json' : _a;
        // method url 置换
        var _methodUrl = methodUrl;
        if (typeof methodUrl === 'function') {
            _methodUrl = methodUrl(config.params);
        }
        var reqConfig = config;
        if (Array.isArray(_methodUrl)) {
            var _b = __read(_methodUrl, 2), cusConfig = _b[0], methodPath = _b[1];
            reqConfig = __assign(__assign({}, reqConfig), cusConfig);
            _methodUrl = methodPath;
        }
        // 置换 方法 和 地址
        // example:
        //  'getUser' => 'get getUser'
        //  'post updateUser'
        var _methodUrlArr = _methodUrl.split(' ');
        var _c = __read(_methodUrlArr.length === 1 ? ['get', _methodUrl] : _methodUrlArr, 2), method = _c[0], urlPath = _c[1];
        // 拼接 prefix (处理prefix 末尾以及 urlPath 开始 / 的重复)
        var url = "".concat(prefix).concat(prefix.endsWith('/') ? '' : '/').concat(urlPath.startsWith('/') ? urlPath.substring(1) : urlPath);
        reqConfig.method = method;
        // getLike 请求把data 参数当成 params 传递
        // const isGetLike = ['get', 'delete', 'head', 'options'].includes(method);
        // reqConfig.params = isGetLike ? { ...reqConfig.data, ...reqConfig.params } : reqConfig.params;
        // headers 默认值设置
        reqConfig.headers = __assign(__assign({}, (REQUEST_HEADERS[requestType] || {})), (reqConfig.headers || {}));
        // 标记为通过 registApi 注册
        reqConfig._registId = id;
        return request(url, reqConfig);
    };
}
//# sourceMappingURL=registApi.js.map